@@ -31,27 +31,31 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <string.h>
 #include <atk/atk.h>
 
 #include <droute/droute.h>
 
 #include "bridge.h"
 #include "event.h"
-#include "accessible-register.h"
 #include "adaptors.h"
 
+#include "accessible-register.h"
+#include "accessible-leasing.h"
+#include "accessible-cache.h"
+
 #include "common/spi-dbus.h"
 
 /*
  * Provides the path for the introspection directory.
  */
 #if !defined ATSPI_INTROSPECTION_PATH
-    #error "No introspection XML directory defined"
+#error "No introspection XML directory defined"
 #endif
 
 /*---------------------------------------------------------------------------*/
 
-SpiAppData *atk_adaptor_app_data = NULL;
+SpiBridge *spi_global_app_data = NULL;
 
 static const AtkMisc *atk_misc = NULL;
 
@@ -64,36 +68,36 @@ static const AtkMisc *atk_misc = NULL;
  * with the screen number stripped off if present.
  *
  */
-static const gchar*
+static const gchar *
 spi_display_name (void)
 {
-    static const char *canonical_display_name = NULL;
-    if (!canonical_display_name)
-      {
-        const gchar *display_env = g_getenv ("AT_SPI_DISPLAY");
-        if (!display_env)
-          {
-            display_env = g_getenv ("DISPLAY");
-            if (!display_env || !display_env[0]) 
-                canonical_display_name = ":0";
-            else
-              {
-                gchar *display_p, *screen_p;
-                canonical_display_name = g_strdup (display_env);
-                display_p = strrchr (canonical_display_name, ':');
-                screen_p = strrchr (canonical_display_name, '.');
-                if (screen_p && display_p && (screen_p > display_p))
-                  {
-                    *screen_p = '\0';
-                  }
-              }
-          }
-        else
-          {
-            canonical_display_name = display_env;
-          }
-      }
-    return canonical_display_name;
+  static const char *canonical_display_name = NULL;
+  if (!canonical_display_name)
+    {
+      const gchar *display_env = g_getenv ("AT_SPI_DISPLAY");
+      if (!display_env)
+        {
+          display_env = g_getenv ("DISPLAY");
+          if (!display_env || !display_env[0])
+            canonical_display_name = ":0";
+          else
+            {
+              gchar *display_p, *screen_p;
+              canonical_display_name = g_strdup (display_env);
+              display_p = strrchr (canonical_display_name, ':');
+              screen_p = strrchr (canonical_display_name, '.');
+              if (screen_p && display_p && (screen_p > display_p))
+                {
+                  *screen_p = '\0';
+                }
+            }
+        }
+      else
+        {
+          canonical_display_name = display_env;
+        }
+    }
+  return canonical_display_name;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -108,59 +112,60 @@ spi_display_name (void)
 static DBusConnection *
 spi_atk_bridge_get_bus (void)
 {
-     Atom AT_SPI_BUS;
-     Atom actual_type;
-     Display *bridge_display;
-     int actual_format;
-     unsigned char *data = NULL;  
-     unsigned long nitems;
-     unsigned long leftover;
+  Atom AT_SPI_BUS;
+  Atom actual_type;
+  Display *bridge_display;
+  int actual_format;
+  unsigned char *data = NULL;
+  unsigned long nitems;
+  unsigned long leftover;
 
-     DBusConnection *bus = NULL;
-     DBusError       error;
+  DBusConnection *bus = NULL;
+  DBusError error;
 
-     bridge_display = XOpenDisplay (spi_display_name ());
-     if (!bridge_display)
-	g_error ("AT_SPI: Could not get the display\n");
+  bridge_display = XOpenDisplay (spi_display_name ());
+  if (!bridge_display)
+    g_error ("AT_SPI: Could not get the display\n");
 
-     AT_SPI_BUS = XInternAtom (bridge_display, "AT_SPI_BUS", False); 
-     XGetWindowProperty(bridge_display, 
-                        XDefaultRootWindow (bridge_display),
-                        AT_SPI_BUS, 0L,
-                        (long)BUFSIZ, False,
-                        (Atom) 31, &actual_type, &actual_format,
-                        &nitems, &leftover, &data);
+  AT_SPI_BUS = XInternAtom (bridge_display, "AT_SPI_BUS", False);
+  XGetWindowProperty (bridge_display,
+                      XDefaultRootWindow (bridge_display),
+                      AT_SPI_BUS, 0L,
+                      (long) BUFSIZ, False,
+                      (Atom) 31, &actual_type, &actual_format,
+                      &nitems, &leftover, &data);
 
-     dbus_error_init (&error);
+  dbus_error_init (&error);
 
-     if (data == NULL)
-     {
-         g_warning ("AT-SPI: Accessibility bus not found - Using session bus.\n");
-         bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
-         if (!bus)
-             g_error ("AT-SPI: Couldn't connect to bus: %s\n", error.message);
-     }
-     else
-     {
-	 bus = dbus_connection_open (data, &error);
-         if (!bus)
-         {
-             g_error ("AT-SPI: Couldn't connect to bus: %s\n", error.message);
-         }
-	 else
-         {
-	     if (!dbus_bus_register (bus, &error))
-	         g_error ("AT-SPI: Couldn't register with bus: %s\n");
-         } 
-     }
+  if (data == NULL)
+    {
+      g_warning
+        ("AT-SPI: Accessibility bus not found - Using session bus.\n");
+      bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
+      if (!bus)
+        g_error ("AT-SPI: Couldn't connect to bus: %s\n", error.message);
+    }
+  else
+    {
+      bus = dbus_connection_open (data, &error);
+      if (!bus)
+        {
+          g_error ("AT-SPI: Couldn't connect to bus: %s\n", error.message);
+        }
+      else
+        {
+          if (!dbus_bus_register (bus, &error))
+            g_error ("AT-SPI: Couldn't register with bus: %s\n", error.message);
+        }
+    }
 
-     return bus;
+  return bus;
 }
 
 /*---------------------------------------------------------------------------*/
 
 static void
-register_application (SpiAppData *app)
+register_application (SpiBridge * app)
 {
   DBusMessage *message;
   DBusMessageIter iter;
@@ -175,22 +180,23 @@ register_application (SpiAppData *app)
                                           "RegisterApplication");
   dbus_message_set_no_reply (message, TRUE);
 
-  uname = dbus_bus_get_unique_name(app->bus);
+  uname = dbus_bus_get_unique_name (app->bus);
   if (!uname)
-  {
+    {
       g_error ("AT-SPI: Couldn't get unique name for this connection");
-  }
+    }
 
-  dbus_message_iter_init_append(message, &iter);
-  dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &uname);
+  dbus_message_iter_init_append (message, &iter);
+  dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &uname);
   dbus_connection_send (app->bus, message, NULL);
-  if (message) dbus_message_unref (message);
+  if (message)
+    dbus_message_unref (message);
 }
 
 /*---------------------------------------------------------------------------*/
 
 static void
-deregister_application (SpiAppData *app)
+deregister_application (SpiBridge * app)
 {
   DBusMessage *message;
   DBusMessageIter iter;
@@ -205,12 +211,13 @@ deregister_application (SpiAppData *app)
                                           "DeregisterApplication");
   dbus_message_set_no_reply (message, TRUE);
 
-  uname = dbus_bus_get_unique_name(app->bus);
+  uname = dbus_bus_get_unique_name (app->bus);
 
-  dbus_message_iter_init_append(message, &iter);
-  dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &uname);
+  dbus_message_iter_init_append (message, &iter);
+  dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &uname);
   dbus_connection_send (app->bus, message, NULL);
-  if (message) dbus_message_unref (message);
+  if (message)
+    dbus_message_unref (message);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -218,17 +225,17 @@ deregister_application (SpiAppData *app)
 static void
 exit_func (void)
 {
-  if (!atk_adaptor_app_data)
+  if (!spi_global_app_data)
     {
       return;
     }
 
   spi_atk_tidy_windows ();
-  spi_atk_deregister_event_listeners();
-  deregister_application (atk_adaptor_app_data);
+  spi_atk_deregister_event_listeners ();
+  deregister_application (spi_global_app_data);
 
-  g_free(atk_adaptor_app_data);
-  atk_adaptor_app_data = NULL;
+  g_free (spi_global_app_data);
+  spi_global_app_data = NULL;
 
   /* Not currently creating an XDisplay */
 #if 0
@@ -239,30 +246,48 @@ exit_func (void)
 
 /*---------------------------------------------------------------------------*/
 
-#ifdef __ATK_PLUG_H__
+#ifdef SPI_ATK_PLUG_SOCKET
 static AtkPlugClass *plug_class;
 static AtkSocketClass *socket_class;
 
 static gchar *
-get_plug_id (AtkPlug *plug)
+get_plug_id (AtkPlug * plug)
 {
-  const char *uname = dbus_bus_get_unique_name(atk_adaptor_app_data->bus);
+  const char *uname = dbus_bus_get_unique_name (spi_global_app_data->bus);
   gchar *path;
   GString *str = g_string_new (NULL);
 
-  path = atk_dbus_object_to_path (ATK_OBJECT(plug), TRUE);
+  path = spi_register_object_to_path (spi_global_register, G_OBJECT (plug));
   g_string_printf (str, "%s:%s", uname, path);
   g_free (path);
   return g_string_free (str, FALSE);
 }
 
 static void
-socket_embed_hook (AtkSocket *socket, gchar *plug_id)
+socket_embed_hook (AtkSocket * socket, gchar * plug_id)
 {
   AtkObject *accessible = ATK_OBJECT(socket);
+  gchar *plug_name, *plug_path;
+
   /* Force registration */
-  gchar *path = atk_dbus_object_to_path (accessible, TRUE);
-  spi_emit_cache_update (accessible, atk_adaptor_app_data->bus);
+  gchar *path = spi_register_object_to_path (spi_global_register, G_OBJECT (accessible));
+  /* Let the plug know that it has been embedded */
+  plug_name = g_strdup (plug_id);
+  if (!plug_name)
+    {
+      g_free (path);
+      return;
+    }
+  plug_path = g_utf8_strchr (plug_name + 1, -1, ':');
+  if (plug_path)
+    {
+      DBusMessage *message;
+      *(plug_path++) = '\0';
+      message = dbus_message_new_method_call (plug_name, plug_path, "org.freedesktop.atspi.Accessible", "Embedded");
+      dbus_message_append_args (message, DBUS_TYPE_STRING, &path, DBUS_TYPE_INVALID);
+      dbus_connection_send (spi_global_app_data->bus, message, NULL);
+    }
+  g_free (plug_name);
   g_free (path);
 }
 
@@ -270,7 +295,7 @@ static void
 install_plug_hooks ()
 {
   gpointer data;
-  
+
   data = g_type_class_ref (ATK_TYPE_PLUG);
   plug_class = ATK_PLUG_CLASS (data);
   data = g_type_class_ref (ATK_TYPE_SOCKET);
@@ -283,10 +308,11 @@ install_plug_hooks ()
 gchar *atspi_dbus_name = NULL;
 static gboolean atspi_no_register = FALSE;
 
-static GOptionEntry atspi_option_entries[] =
-{
-  {"atspi-dbus-name", 0, 0, G_OPTION_ARG_STRING, &atspi_dbus_name, "D-Bus bus name to register as", NULL},
-  {"atspi-no-register", 0, 0, G_OPTION_ARG_NONE, &atspi_no_register, "Do not register with Registry Daemon", NULL},
+static GOptionEntry atspi_option_entries[] = {
+  {"atspi-dbus-name", 0, 0, G_OPTION_ARG_STRING, &atspi_dbus_name,
+   "D-Bus bus name to register as", NULL},
+  {"atspi-no-register", 0, 0, G_OPTION_ARG_NONE, &atspi_no_register,
+   "Do not register with Registry Daemon", NULL},
   {NULL}
 };
 
@@ -301,7 +327,7 @@ static GOptionEntry atspi_option_entries[] =
  *
  */
 static int
-adaptor_init (gint *argc, gchar **argv[])
+adaptor_init (gint * argc, gchar ** argv[])
 {
   GOptionContext *opt;
   GError *err = NULL;
@@ -322,64 +348,78 @@ adaptor_init (gint *argc, gchar **argv[])
   g_return_val_if_fail (root, 0);
 
   /* Parse command line options */
-  opt = g_option_context_new(NULL);
-  g_option_context_add_main_entries(opt, atspi_option_entries, NULL);
-  g_option_context_set_ignore_unknown_options(opt, TRUE);
-  if (!g_option_context_parse(opt, argc, argv, &err))
-      g_warning("AT-SPI Option parsing failed: %s\n", err->message);
+  opt = g_option_context_new (NULL);
+  g_option_context_add_main_entries (opt, atspi_option_entries, NULL);
+  g_option_context_set_ignore_unknown_options (opt, TRUE);
+  if (!g_option_context_parse (opt, argc, argv, &err))
+    g_warning ("AT-SPI Option parsing failed: %s\n", err->message);
 
   /* Allocate global data and do ATK initializations */
-  atk_adaptor_app_data = g_new0 (SpiAppData, 1);
+  spi_global_app_data = g_new0 (SpiBridge, 1);
   atk_misc = atk_misc_get_instance ();
-  atk_adaptor_app_data->root = root;
+  spi_global_app_data->root = g_object_ref (root);
 
   /* Set up D-Bus connection and register bus name */
   dbus_error_init (&error);
-  atk_adaptor_app_data->bus = spi_atk_bridge_get_bus ();
-  if (!atk_adaptor_app_data->bus)
-  {
-    g_free(atk_adaptor_app_data);
-    atk_adaptor_app_data = NULL;
-    return 0;
-  }
+  spi_global_app_data->bus = spi_atk_bridge_get_bus ();
+  if (!spi_global_app_data->bus)
+    {
+      g_free (spi_global_app_data);
+      spi_global_app_data = NULL;
+      return 0;
+    }
 
   if (atspi_dbus_name != NULL)
-  {
-    if (dbus_bus_request_name(atk_adaptor_app_data->bus, atspi_dbus_name, 0, &error))
     {
-      g_print("AT-SPI Recieved D-Bus name - %s\n", atspi_dbus_name);
+      if (dbus_bus_request_name
+          (spi_global_app_data->bus, atspi_dbus_name, 0, &error))
+        {
+          g_print ("AT-SPI Recieved D-Bus name - %s\n", atspi_dbus_name);
+        }
+      else
+        {
+          g_print
+            ("AT-SPI D-Bus name requested but could not be allocated - %s\n",
+             atspi_dbus_name);
+        }
     }
-    else
-    {
-      g_print("AT-SPI D-Bus name requested but could not be allocated - %s\n", atspi_dbus_name);
-    }
-  }
 
-  dbus_connection_setup_with_g_main(atk_adaptor_app_data->bus, g_main_context_default());
+  dbus_connection_setup_with_g_main (spi_global_app_data->bus,
+                                     g_main_context_default ());
+
+  /* 
+   * Create the leasing, register and cache objects.
+   * The order is important here, the cache depends on the
+   * register object.
+   */
+  spi_global_register = g_object_new (SPI_REGISTER_TYPE, NULL);
+  spi_global_leasing  = g_object_new (SPI_LEASING_TYPE, NULL);
+  spi_global_cache    = g_object_new (SPI_CACHE_TYPE, NULL);
 
   /* Get D-Bus introspection directory */
-  introspection_directory = (char *) g_getenv("ATSPI_INTROSPECTION_PATH");
+  introspection_directory = (char *) g_getenv ("ATSPI_INTROSPECTION_PATH");
   if (introspection_directory == NULL)
-      introspection_directory = ATSPI_INTROSPECTION_PATH;
+    introspection_directory = ATSPI_INTROSPECTION_PATH;
 
   /* Register droute for routing AT-SPI messages */
-  atk_adaptor_app_data->droute = droute_new (atk_adaptor_app_data->bus, introspection_directory);
+  spi_global_app_data->droute =
+    droute_new (spi_global_app_data->bus, introspection_directory);
 
-  treepath = droute_add_one (atk_adaptor_app_data->droute,
-                             "/org/freedesktop/atspi/tree",
-                             NULL);
+  treepath = droute_add_one (spi_global_app_data->droute,
+                             "/org/at_spi/cache", spi_global_cache);
 
-  accpath = droute_add_many (atk_adaptor_app_data->droute,
-                             "/org/freedesktop/atspi/accessible",
+  accpath = droute_add_many (spi_global_app_data->droute,
+                             "/org/at_spi/accessible",
                              NULL,
-                             (DRouteGetDatumFunction) atk_dbus_path_to_gobject);
+                             (DRouteGetDatumFunction)
+                             spi_global_register_path_to_object);
+
 
   /* Register all interfaces with droute and set up application accessible db */
-  spi_initialize_tree (treepath);
-
+  spi_initialize_cache (treepath);
   spi_initialize_accessible (accpath);
   spi_initialize_application (accpath);
-  spi_initialize_action(accpath);
+  spi_initialize_action (accpath);
   spi_initialize_collection (accpath);
   spi_initialize_component (accpath);
   spi_initialize_document (accpath);
@@ -392,20 +432,17 @@ adaptor_init (gint *argc, gchar **argv[])
   spi_initialize_text (accpath);
   spi_initialize_value (accpath);
 
-  /* Initialize the AtkObject registration */
-  atk_dbus_initialize (atk_adaptor_app_data->root);
-
   /* Register methods to send D-Bus signals on certain ATK events */
   spi_atk_register_event_listeners ();
 
-#ifdef __ATK_PLUG_H__
+#ifdef SPI_ATK_PLUG_SOCKET
   /* Hook our plug-and socket functions */
   install_plug_hooks ();
 #endif
 
   /* Register this app by sending a signal out to AT-SPI registry daemon */
   if (!atspi_no_register)
-    register_application (atk_adaptor_app_data);
+    register_application (spi_global_app_data);
 
   g_atexit (exit_func);
 
@@ -415,13 +452,13 @@ adaptor_init (gint *argc, gchar **argv[])
 /*---------------------------------------------------------------------------*/
 
 int
-gtk_module_init (gint *argc, gchar **argv[])
+gtk_module_init (gint * argc, gchar ** argv[])
 {
   const gchar *load_bridge = g_getenv ("NO_AT_BRIDGE");
 
   if (!load_bridge || g_ascii_strtod (load_bridge, NULL) == 0)
     {
-	return adaptor_init (argc, argv);
+      return adaptor_init (argc, argv);
     }
   return 0;
 }
@@ -437,7 +474,7 @@ gnome_accessibility_module_init (void)
 
       if (g_getenv ("AT_SPI_DEBUG"))
         {
-	    g_print("Atk Accessibility bridge initialized\n");
+          g_print ("Atk Accessibility bridge initialized\n");
         }
     }
 }
@@ -448,4 +485,5 @@ gnome_accessibility_module_shutdown (void)
   spi_atk_deregister_event_listeners ();
   exit_func ();
 }
+
 /*END------------------------------------------------------------------------*/
